/*
 * TODO license!
 */
package org.netbeans.plugin.output.clear;

import java.awt.event.ActionEvent;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Collection;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.AbstractAction;
import javax.swing.Action;
import org.netbeans.api.annotations.common.StaticResource;
import org.openide.util.ImageUtilities;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.util.lookup.ServiceProvider;
import org.openide.windows.IOContainer;
import org.openide.windows.IOProvider;
import org.openide.windows.InputOutput;
import org.openide.windows.OutputWriter;

/**
 *
 */
@ServiceProvider(service = IOProvider.class, position = 99)
public class WindowWithClearButtonProvider extends IOProvider {

    private static final Logger LOG
            = Logger.getLogger(WindowWithClearButtonProvider.class.getName());

    @StaticResource
    private static final String CLEAR_ICON
            = "org/netbeans/plugin/output/clear/resources/" //NOI18N
            + "clear_icon.png";//NOI18N
    private IOProvider delegate = null;

    private IOProvider getDelegate() {

        if (delegate != null) {
            return delegate;
        }
        Collection<? extends IOProvider> lookupAll
                = Lookup.getDefault().lookupAll(IOProvider.class);
        for (IOProvider prov : lookupAll) {
            if ("org.netbeans.core.output2.NbIOProvider".equals(
                    prov.getClass().getName())) {
                LOG.log(Level.FINE, "Delegate found: {0}", prov);       //NOI18N
                this.delegate = prov;
                return prov;
            }
        }
        throw new IllegalStateException("No delegate NbIOProvider");    //NOI18N
    }

    @Override
    public InputOutput getIO(String name, boolean newIO) {
        return getIO(name, newIO, new Action[0], null);
    }

    @Override
    public InputOutput getIO(String name, Action[] actions) {
        return getIO(name, true, actions, null);
    }

    @Override
    public InputOutput getIO(String name, Action[] actions,
            IOContainer ioContainer) {
        return getIO(name, true, actions, ioContainer);
    }

    @Override
    public InputOutput getIO(String name, boolean newIO, Action[] actions,
            IOContainer ioContainer) {
        Action[] extendedActions = extendActionArray(actions);
        InputOutput io = getDelegate().getIO(name, newIO, extendedActions,
                ioContainer);
        ((ClearAction) extendedActions[0]).setInputOutput(io);
        return io;
    }

    @Override
    public String getName() {
        return getDelegate().getName();
    }

    @Override
    public OutputWriter getStdOut() {
        return getDelegate().getStdOut();
    }

    /**
     * Prepend the actions array with a {@link ClearAction}.
     */
    private Action[] extendActionArray(Action[] actions) {
        Action[] newArray = new Action[actions.length + 1];
        newArray[0] = new ClearAction();
        System.arraycopy(actions, 0, newArray, 1, actions.length);
        return newArray;

    }

    private class ClearAction extends AbstractAction {

        private InputOutput inputOutput = null;

        @NbBundle.Messages({"TXT_Clear=Clear"})
        public ClearAction() {
            putValue(NAME, Bundle.TXT_Clear());
            putValue(SMALL_ICON, ImageUtilities.loadImageIcon(
                    CLEAR_ICON, false));
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            if (inputOutput != null) {
                try {
                    Method writerMethod = inputOutput.getClass()
                            .getDeclaredMethod("writer");               //NOI18N
                    writerMethod.setAccessible(true);
                    Object writer = writerMethod.invoke(inputOutput);
                    Method resetMethod = writer.getClass()
                            .getDeclaredMethod("reset");                //NOI18N
                    resetMethod.setAccessible(true);
                    Method isStreamClosedMethod = inputOutput.getClass()
                            .getDeclaredMethod("isStreamClosed");       //NOI18N
                    isStreamClosedMethod.setAccessible(true);
                    Method setStreamClosedMethod = inputOutput.getClass()
                            .getDeclaredMethod("setStreamClosed", //NOI18N
                                    boolean.class);
                    setStreamClosedMethod.setAccessible(true);
                    boolean isStreamClosed = (Boolean) isStreamClosedMethod
                            .invoke(inputOutput);
                    resetMethod.invoke(writer);
                    setStreamClosedMethod.invoke(inputOutput, isStreamClosed);
                } catch (NoSuchMethodException ex) {
                    LOG.log(Level.WARNING, null, ex);
                } catch (SecurityException ex) {
                    LOG.log(Level.WARNING, null, ex);
                } catch (IllegalAccessException ex) {
                    LOG.log(Level.WARNING, null, ex);
                } catch (IllegalArgumentException ex) {
                    LOG.log(Level.WARNING, null, ex);
                } catch (InvocationTargetException ex) {
                    LOG.log(Level.WARNING, null, ex);
                }
            }
        }

        public void setInputOutput(InputOutput inputOutput) {
            this.inputOutput = inputOutput;
        }
    }
}
